indexing
	description: "An octet stream; i.e. a sequence of unsigned 8 bit integers"
	license: "Eiffel Forumn License v2 - See license.txt"
	source: "{
		Amalasoft Corporation
		273 Harwood Avenue
		Littleton, MA 01460 USA
		}"
	status: "Copyright 1995-2008, Amalasoft";
	date: "$Date: 2008/12/03 $";
	revision: "$Revision: 001$";
	howto: "{
    To use this class, create a new instance with a given number of octets
    using 'make', or from an existing stream using either 'make_from_string'
    or 'make_from_managed_pointer'.
	 N.B. The lower bound is ALWAYS 0 !!!
}"

class AEL_DS_BYTE_STREAM

inherit
	ARRAY [NATURAL_8]
		rename
			make as array_make
		redefine
			subarray
		end

create
	make, make_from_string, make_from_managed_pointer

--|========================================================================
feature {NONE} -- Creation
--|========================================================================

	make (sz: INTEGER) is
			-- Create Current with the 'sz' capacity
		require
			valid_size: sz >= 0
		do
			array_make (0, (sz - 1).max (0))
		end

	--|--------------------------------------------------------------

	make_from_string (v: STRING) is
			-- Create Current from the given string
		require
			exists: v /= Void
		local
			i, j, lim: INTEGER
		do
			make (v.count)
			lim := upper
			from
				i := 0
				j := 1
			until
				i > lim
			loop
				put (v.item (j).code.as_natural_8, i)
				i := i + 1
				j := j + 1
			end
		ensure
			same_as_string: v.is_equal (to_string)
		end

	--|--------------------------------------------------------------

	make_from_managed_pointer (mp: MANAGED_POINTER; sz: INTEGER) is
			-- Create Current with 'sz' capacity, from the managed 
			-- pointer 'mp'
		require
			exists: mp /= Void
			valid_size: sz >= 0
		local
			i: INTEGER
		do
			make (sz)
			from i := 0
			until i >= sz
			loop
				put (mp.read_natural_8 (i), i)
				i := i + 1
			end
		end

--|========================================================================
feature -- Conversion
--|========================================================================

	to_string: STRING is
			-- Content of stream as a string of 8-bit characters
		local
			i: INTEGER
		do
			create Result.make (count)
			from i := 0
			until i > upper
			loop
				Result.extend (item (i).to_character_8)
				i := i + 1
			end
		ensure
			exists: Result /= Void
			same_length: Result.count = count
		end

	--|--------------------------------------------------------------

	truncated_stream (len: INTEGER_32): AEL_DS_BYTE_STREAM is
			-- Sub-stream from start, containing len items or count
			-- items if len is larger than count
		do
			Result := subarray (0, (len - 1).min (upper))
		end

	--|--------------------------------------------------------------

	subarray (spos, epos: INTEGER_32): AEL_DS_BYTE_STREAM is
			-- Sub-stream from spos through epos, inclusive,
			-- where spos and epos are zero-based
		do
			create Result.make ((epos - spos) + 1)
			if spos <= epos then
				Result.subcopy (Current, spos, epos, spos)
			end
		end

	--|--------------------------------------------------------------
invariant
	zero_lower_bound: lower = 0
	zero_based: capacity = (upper + 1)

end -- class AEL_DS_BYTE_STREAM
