indexing
	description: "{
      A class resembling a list, but actually having a list
}"
	license: "Eiffel Forumn License v2 - See license.txt"
	source: "{
		Amalasoft Corporation
		273 Harwood Avenue
		Littleton, MA 01460 USA
		}"
	status: "Copyright 1995-2008, Amalasoft";
	date: "$Date: 2008/12/03 $";
	revision: "$Revision: 001$";
	howto: "{
    Class should be inherited by your own class that defines the actual
    LIST type, creation and other deferred features.
	 Use typical LIST  routines to add, delete and access items.
}"

deferred class AEL_DS_INDIRECT_LIST [G]

--|========================================================================
feature {NONE} -- Initialization
--|========================================================================

	make is
			-- Create Current
		do
			core_make
			make_list
		end

	--|--------------------------------------------------------------

	make_list is
			-- Create the list structure itself
		deferred
		ensure
			list_exists: list /= Void
		end

	--|--------------------------------------------------------------

	core_make is
		do
		end

--|========================================================================
feature -- Status
--|========================================================================

	count: INTEGER is
			-- Number of items in list
		do
			Result := list.count
		end

	is_empty: BOOLEAN is
			-- Is the list empty?
		do
			Result := count = 0
		end

	--|--------------------------------------------------------------

	has (v: like item): BOOLEAN is
			-- Does the list have the given item?
			-- Object or reference; reference by default
		do
			Result := list.has (v)
		end

--|========================================================================
feature -- Access
--|========================================================================

	item: G is
			-- Item at cursor
		do
			Result := list.item
		end

	first: G is
			-- First item in list
		do
			Result := list.first
		end

	last: G is
			-- Last item in list
		do
			Result := list.last
		end

--|========================================================================
feature -- Cursor movement and state
--|========================================================================

	cursor: CURSOR is do Result := list.cursor end
			-- List cursor

	valid_cursor (v: like cursor): BOOLEAN is
			-- Is the given cursor valid?
		do
			Result := list.valid_cursor (v)
		end

	start is do list.start end
			-- Set the list cursor to the start of the list

	finish is do list.finish end
			-- Set the list cursor to the end of the list

	forth is do list.forth end
			-- Advance the list cursor one position

	back is do list.back end
			-- Retreat the list cursor one position

	after: BOOLEAN is do Result := list.after end
			-- Is the list cursor past the end?

	exhausted: BOOLEAN is do Result := list.exhausted end
			-- Is the list exhausted?

	go_to (oc: CURSOR) is do list.go_to (oc) end
			-- Reset the list cursor to 'oc'

	go_i_th (v: INTEGER) is do list.go_i_th (v) end
			-- Reset the list cursor to position 'v'

	search (v: G) is do list.search (v) end
			-- Search for item 'v' (object or reference) from current 
			-- position

--|========================================================================
feature -- Element change
--|========================================================================

	extend (v: G) is
			-- Add item 'v' to end of list
		do
			list.extend (v)
		end

--|========================================================================
feature -- Element removal
--|========================================================================

	wipe_out is
			-- Remove all items
		do
			list.wipe_out
		end

	--|--------------------------------------------------------------

	remove is
			-- Remove item at cursor
		do
			list.remove
		end

--|========================================================================
feature -- Status setting
--|========================================================================

	compare_objects is
			-- Use object comparson for searches
		do
			list.compare_objects
		end

	compare_references is
			-- Use reference comparson for searches
		do
			list.compare_references
		end

--|========================================================================
feature {AEL_DS_INDIRECT_LIST} -- Structure
--|========================================================================

	list: LIST [G]
			-- Actual list object

	--|--------------------------------------------------------------
invariant
	list_exists: list /= Void

end -- class AEL_DS_INDIRECT_LIST


